----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    20:48:06 03/22/2011 
-- Design Name: 
-- Module Name:    DDR2_Read_VHDL - Verhalten 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

	--------------------------------------------
	-- Beschreibung :
	--
	-- Die State-Machine liest
	-- einen 64Bit Wert aus dem RAM
	-- von 4 aufeinander folgenden 16bit-Zellen
	--
	-- die Adresse wird
	-- von der bergeordneten CONTROL-Unit
	-- gehandelt
	-- 
	-- solange die Read-Funktion luft,
	-- ist BUSY=1
	--------------------------------------------

entity DDR2_Read_VHDL is

	--------------------------------------------
	-- Port Deklerationen
	--------------------------------------------
	port (
		reset_in : in std_logic;
		clk_in : in std_logic;
		clk90_in : in std_logic;
		r_command_register : out std_logic_vector(2 downto 0);
		r_cmd_ack : in std_logic;
		r_burst_done : out std_logic;
		r_data_valid : in std_logic;
		read_en : in std_logic;
		read_busy : out std_logic;
		output_data : in std_logic_vector(31 downto 0);
		read_data : out std_logic_vector(63 downto 0)
	);
	
end DDR2_Read_VHDL;

architecture Verhalten of DDR2_Read_VHDL is

	--------------------------------------------
	-- Interne Signale
	--------------------------------------------
	
	signal v_data_lsb : std_logic_vector(31 downto 0);
	signal v_data_msb : std_logic_vector(31 downto 0);
	
	constant clk_anz : integer := 1; -- warte 2 Clockzyklen (1 bis 0 = 0)
	signal v_counter :  natural range 0 to clk_anz := clk_anz;
	
	type STATE_RA_TYPE is (
		RA_0_NOP,
		RA_1_WAIT_4_ACK1,
		RA_2_WAIT_2_CLK,
		RA_3_SET_NOP,
		RA_4_WAIT_4_ACK0
	);
	signal STATE_RA : STATE_RA_TYPE := RA_0_NOP;

	type STATE_RB_TYPE is (
		RB_0_NOP,
		RB_1_WAIT_4_ACK1,
		RB_2_WAIT_4_VALID1,
		RB_3_DATA_MSB
	);
	signal STATE_RB : STATE_RB_TYPE := RB_0_NOP;	

begin

	-----------------------------------------
	-- State-Machine RA
	-- Clock-0 Lo-Flanke
	-----------------------------------------	
	P_Read_RA : process(clk_in,reset_in)
	begin
		if reset_in = '1' then
			-- reset button ist gedrueckt
			STATE_RA <= RA_0_NOP;
			r_command_register <= "000"; -- NOP
			r_burst_done <= '0';										
		elsif falling_edge(clk_in) then
			case STATE_RA is
				when RA_0_NOP =>
					-- NOP-CMD anlegen
					-- warten auf read enable signal
					r_command_register <= "000";
					v_counter <= clk_anz;
					r_burst_done <= '0';					
					if read_en = '1' then
						-- read enable wurde erkannt
						STATE_RA <= RA_1_WAIT_4_ACK1;
					end if;			
				when RA_1_WAIT_4_ACK1 =>
					-- READ-CMD anlegen
					-- warten auf ACK=1 signal
					-- dieses Signal wird von STATE_RB gemessen
					r_command_register <= "110";
					if STATE_RB = RB_2_WAIT_4_VALID1 then
						-- ack-signal wurde erkannt
						STATE_RA <= RA_2_WAIT_2_CLK;
					end if;
				when RA_2_WAIT_2_CLK =>
					-- warte bis 2 Clockzyklen um sind
					if v_counter = 0 then
						-- burst_done auf Hi
						r_burst_done <= '1';
						STATE_RA <= RA_3_SET_NOP;
					else
						r_burst_done <= '0';
						v_counter <= v_counter - 1;
					end if;
				when RA_3_SET_NOP =>
					-- NOP-CMD anlegen
					r_command_register <= "000"; -- NOP
					STATE_RA <= RA_4_WAIT_4_ACK0;
				when RA_4_WAIT_4_ACK0 =>
					-- burst_done auf Lo
					-- warten auf ACK=0 signal
					r_burst_done <= '0';			
					if r_cmd_ack = '0' then
						-- ack-signal wurde erkannt
						STATE_RA <= RA_0_NOP;
					end if;					
				when others =>
					NULL;
			end case;
		end if;
	end process P_Read_RA;

	-----------------------------------------
	-- State-Machine RB
	-- Clock-90 Lo-Flanke
	-----------------------------------------	
	P_Read_RB : process(clk90_in,reset_in)
	begin
		if reset_in = '1' then
			-- reset button ist gedrueckt
			STATE_RB <= RB_0_NOP;		
		elsif falling_edge(clk90_in) then
			case STATE_RB is
				when RB_0_NOP =>	
					-- warten auf read enable signal
					-- dieses Signal wird von STATE_RA gemessen
					if STATE_RA = RA_1_WAIT_4_ACK1 then
						-- read enable wurde erkannt
						STATE_RB <= RB_1_WAIT_4_ACK1;
					end if;
				when RB_1_WAIT_4_ACK1 =>
					-- warten auf ACK=1 vom RAM
					if r_cmd_ack = '1' then
						-- ack-signal wurde erkannt
						STATE_RB <= RB_2_WAIT_4_VALID1;
					end if;
				when RB_2_WAIT_4_VALID1 =>					
					-- warte bis LSB Daten gueltig											
					if r_data_valid = '1' then
						-- LSB daten sind ok							
						v_data_lsb <= output_data;								
					  	STATE_RB <= RB_3_DATA_MSB;
					else
						v_data_lsb <= (others => '0'); -- wichtig
					end if;
				when RB_3_DATA_MSB =>
					if r_data_valid = '1' then
						-- MSB daten sind ok	
						v_data_msb <= output_data;
					else
						v_data_msb <= (others => '0'); -- wichtig
					end if;
					STATE_RB <= RB_0_NOP;
				when others =>
					NULL;
			end case;			
		end if;
	end process P_Read_RB;

	
	-----------------------------------------
	-- Read-Busy erzeugen
	-----------------------------------------		
	read_busy <= '0' when STATE_RA=RA_0_NOP else '1';

	-----------------------------------------
	-- Read-Data uebergeben
	-----------------------------------------		
	read_data <= v_data_msb & v_data_lsb when STATE_RB=RB_0_NOP else (others => '0');

end Verhalten;

